PROVIDERS = {
    'twitter': {
        'request_token_url': 'http://twitter.com/oauth/request_token',
        'access_token_url': 'http://twitter.com/oauth/access_token',
        'authenticate_url': 'http://twitter.com/oauth/authenticate',
    },
}
