<html>
    <body>
        <div>
            {% if user.is_authenticated %}
            <a href="{% url xauth_logout %}">Log Out</a> ({{ user }}) /
            <a href="{% url xauth_identity_list %}">Identity List</a>
            {% else %}
            <a href="{% url xauth_login %}">Sign In</a>
            {% endif %}
        </div>
        {% for message in messages %}
        <h3><i>{{ message }}</i></h3>
        {% endfor %}
        {% block content %}{% endblock %}
    </body>
</html>
