{% extends "xauth/base.html" %}
{% load i18n %}

{% block xauth_content %}
<h2>{% trans "Log in to the site using your favourite service" %}</h2>
<ul>
    <li><a href="{% url xauth_prepare "openid" %}?openid_url=https://www.google.com/accounts/o8/id">Google</a></li>
    <li><a href="{% url xauth_prepare "openid" %}?openid_url=http://openid.yandex.ru">Yandex</a></li>
    <li><form method="get" action="{% url xauth_prepare "openid" %}"><label>OpenID:</label><input type="text" name="openid_url" size="20" /><input type="submit" /></form></li>
    <li><a href="{% url xauth_prepare "oauth" %}?provider=twitter">Twitter</a></li>
</ul>
{% endblock %}
