{% extends 'base.html' %}

{% block content %}
<h1>Welcome {{ user }}</h1>
<p>Congratulations! You are now the member of our community. What are you waiting for? Generate content: more and more.</p>
{% endblock %}
