{% extends "databrowse/base_site.html" %}

{% block title %}{{ model.verbose_name_plural|capfirst }}{% endblock %}

{% block content %}

<div id="breadcrumbs"><a href="{{ root_url }}">Home</a> / {{ model.verbose_name_plural|capfirst }}</div>

<h1>{{ model.objects.count }} {% if model.objects.count|pluralize %}{{ model.verbose_name_plural }}{% else %}{{ model.verbose_name }}{% endif %}</h1>

{{ plugin_html }}

<ul class="objectlist">
{% for object in model.objects %}
    <li class="{% cycle 'odd' 'even' %}"><a href="{{ object.url }}">{{ object }}</a></li>
{% endfor %}
</ul>

{% endblock %}
