{% extends 'base.html' %}
{% load i18n %}

{% block content %}
<h1>{% trans "List of all identities associated with you account" %}</h1>
<table>
    <thead>
        <tr>
            <th>{% trans "Date" %}</th>
            <th>{% trans "Identity" %}</th>
            <th>{% trans "Service" %}</th>
        </tr>
    </thead>
    {% for assoc in idents %}
    <tr>
        <td>{{ assoc.created|date }}</td>
        <td>{{ assoc.identity|urlize }}</td>
        <td>{{ assoc.service }}</td>
    </tr>
    {% endfor %}
</table>
{% if not idents %}
<p>{% trans "No identitites." %}</p>
{% endif %}

<p>You can associate <a href="{% url xauth_login %}">more identities</a>.</p>

{% endblock %}
