{% extends "xauth/base.html" %}
{% load i18n %}

{% block xauth_content %}
<h1>{% trans "Complete the Sign Up" %}</h1>
<p>{% trans "To complete the Sign Up please provide following details" %}:</p>
<form method="post">
    {% csrf_token %}
    {{ form.as_p }}
    <p><input type="submit" value="{% trans "Submit" %}" /></p>
</form>
{% endblock %}
